﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using PokerEngine.Betting;
using PokerEngine.Engine;

namespace PokerEngine.Debug
{
    /// <summary>
    /// A helper class which is used in debug mode to verify there are no bugs :)
    /// </summary>
    /// <remarks>
    /// This is a "defensive" approach to prove the correctness of the program instead of writing unit tests.
    /// </remarks>
    internal static class Invariant
    {
#if DEBUG
        /// <summary>
        /// Verifies that the player contributed equally to the pot in the given engine game.
        /// </summary>
        /// <param name="engine">The containing engine</param>
        /// <param name="player">The player to verify contribution</param>
        /// <param name="pot">The engine pot</param>
        internal static void VerifyPotIsEven(BaseEngine engine, Player player, SidePot pot)
        {
            // check that the player didn't fold, and the player has a debt to the pot
            if (engine.PlayingPlayers.Contains(player) &&
                pot.GetPlayerCallSum(player) != 0 &&
                player.Money > 0) // if the player has 0 money, the player is "all in"
            {
                Break(string.Format("Player {0} needs to add money to the pot.", player));
            }
        }

        /// <summary>
        /// A helper method which either breaks the debugger or throws an exception
        /// </summary>
        /// <param name="message">The error message</param>
        private static void Break(string message)
        {
            if (Debugger.Launch())
                Debugger.Break();
            else
                throw new InvalidOperationException(message);
        }

        /// <summary>
        /// Verifies that the pot is event between all playing players
        /// </summary>
        /// <param name="engine">The engine which runs the game</param>
        /// <param name="pot">The engine sidepot</param>
        internal static void VerifyPotIsEven(BaseEngine engine, SidePot pot)
        {
            // placeholder for maximal values
            Player lastMaxPlayer = null;
            int maximalBet = 0;

            foreach (Player cur in engine.PlayingPlayers)
            {
                // check pot is even foreach player
                VerifyPotIsEven(engine, cur, pot);
                // in addition, test player total bet
                int curTotalBet = pot.GetPlayerTotalBet(cur);
                if (curTotalBet > maximalBet)
                {
                    // if the last max player had money but found a player with more money, the last one needs to add money
                    if (lastMaxPlayer != null && lastMaxPlayer.Money > 0)
                    {
                        Break(string.Format("Player {0} must increase bet sum.", lastMaxPlayer));
                    }
                    // replace the maximals
                    maximalBet = curTotalBet;
                    lastMaxPlayer = cur;
                }
            }

        }

        /// <summary>
        /// Verifies that the player contributed to the pot or isn't marked as participating
        /// </summary>
        /// <param name="pot">The pot to verify</param>
        /// <param name="player">The player to check</param>
        internal static void VerifyPlayerParticipation(Pot pot, Player player)
        {
            // assure that a player marked as participating doesn't have a debt to the pot
            if (pot.GetPlayerCallSum(player) > player.Money &&
                pot.HasPlayerParticipated(player))
            {
                Break("Player is marked as participating but needs to add money to the pot");
            }
        }

        /// <summary>
        /// Checks the pot split. Verifies that a winner does not earn more than the winner bets.
        /// </summary>
        /// <param name="pot">The pot which is splitted</param>
        /// <param name="winners">A collection of winners. They all must contribute the same amount of money to the pot</param>
        internal static void CheckPlayerSplit(Pot pot, ICollection<Player> winners)
        {
            // get first winner
            IEnumerator<Player> enumerator = winners.GetEnumerator();
            enumerator.MoveNext();
            // this is the amount which all winners must contribute
            int equalAmount = pot.PlayerToTotalBet[enumerator.Current];
            while (enumerator.MoveNext())
            {
                // this winner contributed a different amount
                if (pot.PlayerToTotalBet[enumerator.Current] != equalAmount)
                    Break("not all of the participating players in the split contributed the same amount of money.");
            }
            // verify that there is no player which placed more money in the pot than the winner
            foreach (KeyValuePair<Player, int> pair in pot.PlayerToTotalBet)
            {
                if (pair.Value > equalAmount)
                    Break(string.Format("player {0} has given more money to the pot then the winning players.", pair.Key));
            }
        }

        /// <summary>
        /// Checks that the engine didn't lose any of the money in the game.
        /// </summary>
        /// <param name="baseEngine">The engine to check</param>
        internal static void CheckMoneySum(BaseEngine baseEngine)
        {
            // sum the players money
            int calculatedSum = baseEngine.Players.Sum((cur) => cur.Money);
            // verify that it is equal to the starting total amount of money
            if (calculatedSum != baseEngine.TotalMoney)
                Break(string.Format("Expected amount of money should be {0}, but is {1}", baseEngine.TotalMoney, calculatedSum));
        }

        /// <summary>
        /// Checks that all of the players which contributed to the pot has the same total bet value
        /// </summary>
        /// <param name="pot">The pot to check</param>
        internal static void VerifyPotIsEven(Pot pot)
        {
            if (pot.PlayerToTotalBet.Count > 0)
            {
                // get the bets of the player ordered by their bets
                var orderedBets = from cur in pot.PlayerToTotalBet
                                  orderby cur.Value descending
                                  select cur;
                // mark the maximal sum
                int maxSum = orderedBets.First().Value;

                foreach (var cur in orderedBets)
                {
                    Player curPlayer = cur.Key;
                    int totalBet = cur.Value;
                    // check that the current bet is lower than the maximal bet, if so the player must be "all in"
                    if (totalBet < maxSum && curPlayer.Money > 0 && pot.ParticipatedPlayers.Contains(curPlayer))
                        Break(string.Format("Pot is un-even. Player {0} can & should contribute more.", cur.Key));
                }

            }
        }

        /// <summary>
        /// Checks that all of the playing players doesn't have a debt to the pot
        /// </summary>
        /// <param name="playingPlayers"></param>
        /// <param name="pot"></param>
        internal static void VerifyPlaingPlayersBet(List<Player> playingPlayers, SidePot pot)
        {
            foreach (Player player in playingPlayers)
            {
                if (pot.GetPlayerCallSum(player) > 0 && player.Money > 0)
                    Break(string.Format("Pot is un-even. Player {0} can & should contribute more.", player));
            }
        }
#else
        /// <summary>
        /// Does nothing in release
        /// </summary>
        /// <param name="pot"></param>
        /// <param name="player"></param>
        [Conditional("DEBUG")]
        internal static void VerifyPlayerParticipation(Pot pot, Player player)
        {
            
        }

        /// <summary>
        /// Does nothing in release
        /// </summary>
        /// <param name="pot"></param>
        [Conditional("DEBUG")]
        internal static void VerifyPotIsEven(Pot pot)
        {

        }

        /// <summary>
        /// Does nothing in release
        /// </summary>
        /// <param name="pot"></param>
        /// <param name="players"></param>
        [Conditional("DEBUG")]
        internal static void CheckPlayerSplit(Pot pot, ICollection<Player> players)
        {
            
        }
        /// <summary>
        /// Does nothing in release
        /// </summary>
        /// <param name="baseEngine"></param>
        [Conditional("DEBUG")]
        internal static void CheckMoneySum(BaseEngine baseEngine)
        {
            
        }

        /// <summary>
        /// Does nothing in release
        /// </summary>
        /// <param name="baseEngine"></param>
        /// <param name="player"></param>
        /// <param name="pot"></param>
        [Conditional("DEBUG")]
        internal static void VerifyPotIsEven(BaseEngine baseEngine, Player player, SidePot pot)
        {
            
        }

        /// <summary>
        /// Does nothing in release
        /// </summary>
        /// <param name="baseEngine"></param>
        /// <param name="pot"></param>
        [Conditional("DEBUG")]
        internal static void VerifyPotIsEven(BaseEngine baseEngine, SidePot pot)
        {
            
        }
        /// <summary>
        /// Does nothing in release
        /// </summary>
        /// <param name="playingPlayers"></param>
        /// <param name="pot"></param>
        [Conditional("DEBUG")]
        internal static void VerifyPlaingPlayersBet(List<Player> playingPlayers, SidePot pot)
        {

        }

#endif
    }
}
